package com.framsticks.params;

import org.apache.log4j.Logger;

import com.framsticks.params.annotations.FramsClassAnnotation;
import com.framsticks.util.DoubleMap;
import com.framsticks.util.FramsticksException;

import java.util.Set;

/**
 * Author: Piotr Śniegowski
 */
public class Registry {
	private static final Logger log = Logger.getLogger(Registry.class.getName());

	// protected void internalRegisterClass(FramsClass framsClass, @Nullable Class<?> javaClass) {
	// 	assert framsClass.getName() != null;
	// 	assert framsClass.getId() != null;
	// 	infoCacheByName.put(framsClass.getName(), framsClass);
	// 	infoCacheById.put(framsClass.getId(), framsClass);

	// 	if (javaClass != null) {
	// 		reflectedClasses.put(framsClass, javaClass);
	// 		infoCacheByJavaName.put(javaClass.getCanonicalName(), framsClass);
	// 	}
	// }
	protected final DoubleMap<String, Class<?>> javaClasses = new DoubleMap<>();
	protected final DoubleMap<String, FramsClass> framsClasses = new DoubleMap<>();

	public void registerReflectedClass(String name, String id, Class<?> reflectedClass) {
		javaClasses.put(id, name, reflectedClass);
	}

	public Registry registerAndBuild(Class<?> reflectedClass) {
		register(reflectedClass);
		putInfoIntoCache(FramsClass.build().forClass(reflectedClass));
		return this;
	}

	public Registry register(Class<?> reflectedClass) {
		FramsClassAnnotation a = reflectedClass.getAnnotation(FramsClassAnnotation.class);
		if (a == null) {
			throw new FramsticksException().msg("class is not annotated").arg("class", reflectedClass);
		}

		registerReflectedClass(FramsClassBuilder.getName(a, reflectedClass), FramsClassBuilder.getId(a, reflectedClass), reflectedClass);
		return this;
	}

	public AccessInterface createAccess(String name, FramsClass framsClass) throws ConstructionException {
		// assert framsClasses.containsValue(framsClass);
		if (javaClasses.containsKey(name)) {
			return new ReflectionAccess(javaClasses.get(name), framsClass);
		}
		return new PropertiesAccess(framsClass);
	}

	public void putInfoIntoCache(FramsClass framsClass) {
		log.debug("caching info for " + framsClass);
		framsClasses.put(framsClass.getId(), framsClass.getName(), framsClass);
	}

	public FramsClass getInfoFromCache(String identifier) {
		if (identifier == null) {
			return null;
		}
		return framsClasses.get(identifier);
	}

	public static AccessInterface wrapAccessWithListIfNeeded(CompositeParam param, AccessInterface access) {
		if (access == null) {
			return null;
		}
		return param.prepareAccessInterface(access);
	}

	public AccessInterface prepareAccess(CompositeParam param) throws ConstructionException {
		return wrapAccessWithListIfNeeded(param, createAccess(param.getContainedTypeName()));
	}

	public AccessInterface createAccess(String name) throws ConstructionException {
		if (name == null) {
			return null;
		}
		FramsClass framsClass = getInfoFromCache(name);
		if (framsClass == null) {
			return null;
		}

		return createAccess(name, framsClass);
	}

	public Set<Class<?>> getReflectedClasses() {
		return javaClasses.getValues();
	}

	public Set<FramsClass> getInfoCache() {
		return framsClasses.getValues();
	}

}
