package com.framsticks.portals;


import com.framsticks.core.AbstractInstanceListener;
import com.framsticks.core.Instance;
import com.framsticks.core.Path;
import com.framsticks.params.annotations.FramsClassAnnotation;
import com.framsticks.params.annotations.ParamAnnotation;
import com.framsticks.util.Logging;
import com.framsticks.util.dispatching.Dispatching;
import com.framsticks.util.dispatching.Future;
import com.framsticks.util.dispatching.JoinableCollection;
import com.framsticks.util.dispatching.RunAt;

import org.apache.log4j.Logger;


/**
 * @author Piotr Sniegowski
 */
@FramsClassAnnotation
public class Portal extends JoinableCollection<Instance> {

	private final static Logger log = Logger.getLogger(Portal.class.getName());

	@ParamAnnotation
	public Integer counter = 0;

	public Portal() {
	}

	// @Override
	// public void run() {
	// 	super.run();
	// 	new PeriodicTask<Portal>(this, 1000) {

	// 		@Override
	// 		public void run() {
	// 			++counter;
	// 			log.debug("counter is now: " + counter);
	// 			again();
	// 		}
	// 	};
	// }

	@Override
	public void add(final Instance instance) {
		super.add(instance);
		instance.addListener(new AbstractInstanceListener() {
			@Override
			public void onRun(Exception e) {
				assert Dispatching.isThreadSafe();

				super.onRun(e);

				if (e != null) {
					return;
				}
				final String path = "/simulator/genepools/groups/0/genotypes";
				instance.invokeLater(new RunAt<Instance>() {
					@Override
					public void run() {
						instance.resolve(path, new Future<Path>() {
							@Override
							public void result(Path result, Exception e) {
								Logging.log(log, "resolve", path, e);
							}
						});
					}
				});
			}
		});
	}

	@ParamAnnotation(id = "counter_squared", name = "Counter Squared")
	public Double getCounterSquared() {
		return (double)(counter * counter);
	}

}
